@@ -4,13 +4,13 @@
 from .error_analysis_constants import ErrorAnalysisDashboardInterface
 from .explanation_constants import (ExplanationDashboardInterface,
                                     WidgetRequestResponseConstants)
-from scipy.sparse import issparse
 import numpy as np
 import pandas as pd
 import traceback
 from .constants import ModelTask, SKLearn
 from .error_handling import _format_exception
 from responsibleai.serialization_utilities import serialize_json_safe
+from responsibleai._input_processing import _convert_to_list
 from erroranalysis._internal.error_analyzer import (
     ModelAnalyzer, PredictionsAnalyzer)
 from erroranalysis._internal.metrics import metric_to_func
@@ -34,10 +34,13 @@ class ErrorAnalysisDashboardInput:
             categorical_features,
             true_y_dataset,
             pred_y,
+            pred_y_dataset,
             model_task,
             metric,
             max_depth,
-            num_leaves):
+            num_leaves,
+            min_child_samples,
+            sample_dataset):
         """Initialize the ErrorAnalysis Dashboard Input.
 
         :param explanation: An object that represents an explanation.
@@ -58,7 +61,7 @@ class ErrorAnalysisDashboardInput:
         :type dataset: numpy.array or list[][] or pandas.DataFrame
         :param true_y: The true labels for the provided explanation.
             Will overwrite any set on explanation object already.
-        :type true_y: numpy.array or list[]
+        :type true_y: numpy.array or list[] or pandas.Series
         :param classes: The class names.
         :type classes: numpy.array or list[]
         :param features: Feature names.
@@ -68,11 +71,16 @@ class ErrorAnalysisDashboardInput:
         :param true_y_dataset: The true labels for the provided dataset.
         Only needed if the explanation has a sample of instances from the
         original dataset.  Otherwise specify true_y parameter only.
-        :type true_y_dataset: numpy.array or list[]
+        :type true_y_dataset: numpy.array or list[] or pandas.Series
         :param pred_y: The predicted y values, can be passed in as an
             alternative to the model and explanation for a more limited
             view.
-        :type pred_y: numpy.ndarray or list[]
+        :type pred_y: numpy.ndarray or list[] or pandas.Series
+        :param pred_y_dataset: The predicted labels for the provided dataset.
+            Only needed if providing a sample dataset for the UI while using
+            the full dataset for the tree view and heatmap. Otherwise specify
+            pred_y parameter only.
+        :type pred_y_dataset: numpy.array or list[] or pandas.Series
         :param model_task: Optional parameter to specify whether the model
             is a classification or regression model. In most cases, the
             type of the model can be inferred based on the shape of the
@@ -82,8 +90,13 @@ class ErrorAnalysisDashboardInput:
         :type model_task: str
         :param metric: The metric name to evaluate at each tree node or
             heatmap grid.  Currently supported classification metrics
-            include 'error_rate', 'recall_score', 'precision_score',
-            'f1_score', and 'accuracy_score'. Supported regression
+            include 'error_rate', 'recall_score' for binary
+            classification and 'micro_recall_score' or
+            'macro_recall_score' for multiclass classification,
+            'precision_score' for binary classification and
+            'micro_precision_score' or 'macro_precision_score'
+            for multiclass classification, 'f1_score',
+            and 'accuracy_score'. Supported regression
             metrics include 'mean_absolute_error', 'mean_squared_error',
             'r2_score', and 'median_absolute_error'.
         :type metric: str
@@ -93,6 +106,15 @@ class ErrorAnalysisDashboardInput:
         :param num_leaves: The number of leaves of the surrogate tree
             trained on errors.
         :type num_leaves: int
+        :param min_child_samples: The minimal number of data required
+            to create one leaf.
+        :type min_child_samples: int
+        :param sample_dataset: Dataset with fewer samples than the main
+            dataset. Used to improve performance only when an
+            Explanation object is not provided.  Used only if
+            explanation is not specified for the dataset explorer.
+            Specify less than 10k points for optimal performance.
+        :type sample_dataset: pd.DataFrame or numpy.ndarray or list[][]
         """
         self._model = model
         full_dataset = dataset
@@ -100,6 +122,10 @@ class ErrorAnalysisDashboardInput:
             full_true_y = true_y
         else:
             full_true_y = true_y_dataset
+        if pred_y_dataset is None:
+            full_pred_y = pred_y
+        else:
+            full_pred_y = pred_y_dataset
         self._categorical_features = categorical_features
         self._string_ind_data = None
         self._categories = []
@@ -113,6 +139,7 @@ class ErrorAnalysisDashboardInput:
         feature_length = None
         self._max_depth = max_depth
         self._num_leaves = num_leaves
+        self._min_child_samples = min_child_samples
 
         if has_explanation:
             if classes is None:
@@ -128,9 +155,11 @@ class ErrorAnalysisDashboardInput:
                 raise ValueError(
                     "Exceeds maximum number of rows"
                     "for visualization (100000)")
+        elif sample_dataset is not None:
+            dataset = sample_dataset
 
         if classes is not None:
-            classes = self._convert_to_list(classes)
+            classes = _convert_to_list(classes)
             self.dashboard_input[
                 ExplanationDashboardInterface.CLASS_NAMES
             ] = classes
@@ -140,7 +169,7 @@ class ErrorAnalysisDashboardInput:
             self._dataframeColumns = dataset.columns
             self._dfdtypes = dataset.dtypes
         try:
-            list_dataset = self._convert_to_list(dataset)
+            list_dataset = _convert_to_list(dataset)
         except Exception as ex:
             ex_str = _format_exception(ex)
             raise ValueError(
@@ -190,7 +219,7 @@ class ErrorAnalysisDashboardInput:
             ] = self._is_classifier
 
         if true_y is not None and len(true_y) == row_length:
-            list_true_y = self._convert_to_list(true_y)
+            list_true_y = _convert_to_list(true_y)
             # If classes specified, convert true_y to numeric representation
             if classes is not None and list_true_y[0] in class_to_index:
                 for i in range(len(list_true_y)):
@@ -200,7 +229,7 @@ class ErrorAnalysisDashboardInput:
             ] = list_true_y
 
         if features is not None:
-            features = self._convert_to_list(features)
+            features = _convert_to_list(features)
             if feature_length is not None and len(features) != feature_length:
                 raise ValueError("Feature vector length mismatch:"
                                  " feature names length differs"
@@ -216,7 +245,7 @@ class ErrorAnalysisDashboardInput:
                                  " for given dataset type,"
                                  " inner error: {}".format(ex_str))
             try:
-                probability_y = self._convert_to_list(probability_y)
+                probability_y = _convert_to_list(probability_y)
             except Exception as ex:
                 ex_str = _format_exception(ex)
                 raise ValueError(
@@ -238,7 +267,7 @@ class ErrorAnalysisDashboardInput:
             # Assume classification for backwards compatibility
             if model_task == ModelTask.UNKNOWN:
                 model_task = ModelTask.CLASSIFICATION
-            self._error_analyzer = PredictionsAnalyzer(pred_y,
+            self._error_analyzer = PredictionsAnalyzer(full_pred_y,
                                                        full_dataset,
                                                        full_true_y,
                                                        features,
@@ -260,11 +289,9 @@ class ErrorAnalysisDashboardInput:
                 metric = Metrics.MEAN_SQUARED_ERROR
             else:
                 metric = self._error_analyzer.metric
-        if model_available and true_y_dataset is not None:
-            full_predicted_y = self.compute_predicted_y(model, full_dataset)
-        else:
-            full_predicted_y = predicted_y
-        self.set_root_metric(full_predicted_y, full_true_y, metric)
+        if model_available:
+            full_pred_y = self.compute_predicted_y(model, full_dataset)
+        self.set_root_metric(full_pred_y, full_true_y, metric)
 
     def set_root_metric(self, predicted_y, true_y, metric):
         if metric != Metrics.ERROR_RATE:
@@ -281,7 +308,7 @@ class ErrorAnalysisDashboardInput:
                     true_y = np.array(true_y)
                 diff = predicted_y != true_y
                 error = sum(diff)
-                metric_value = error / total
+                metric_value = (error / total) * 100
         metric_name = metric_to_display_name[metric]
         root_stats = {
             ExplanationDashboardInterface.ROOT_METRIC_NAME: metric_name,
@@ -310,7 +337,7 @@ class ErrorAnalysisDashboardInput:
 
     def predicted_y_to_list(self, predicted_y):
         try:
-            predicted_y = self._convert_to_list(predicted_y)
+            predicted_y = _convert_to_list(predicted_y)
         except Exception as ex:
             ex_str = _format_exception(ex)
             raise ValueError(
@@ -350,13 +377,13 @@ class ErrorAnalysisDashboardInput:
 
         if local_explanation is not None:
             try:
-                local_explanation["scores"] = self._convert_to_list(
+                local_explanation["scores"] = _convert_to_list(
                     local_explanation["scores"])
                 if np.shape(local_explanation["scores"])[-1] > 1000:
                     raise ValueError("Exceeds maximum number of features for "
                                      "visualization (1000). Please regenerate"
                                      " the explanation using fewer features.")
-                local_explanation["intercept"] = self._convert_to_list(
+                local_explanation["intercept"] = _convert_to_list(
                     local_explanation["intercept"])
                 # We can ignore perf explanation data.
                 # Note if it is added back at any point,
@@ -393,10 +420,10 @@ class ErrorAnalysisDashboardInput:
                                      "local explanations dimension")
         if local_explanation is None and global_explanation is not None:
             try:
-                global_explanation["scores"] = self._convert_to_list(
+                global_explanation["scores"] = _convert_to_list(
                     global_explanation["scores"])
                 if 'intercept' in global_explanation:
-                    global_explanation["intercept"] = self._convert_to_list(
+                    global_explanation["intercept"] = _convert_to_list(
                         global_explanation["intercept"])
                 self.dashboard_input[
                     ExplanationDashboardInterface.GLOBAL_EXPLANATION
@@ -420,7 +447,8 @@ class ErrorAnalysisDashboardInput:
             tree = self._error_analyzer.compute_error_tree(
                 features, filters, composite_filters,
                 max_depth=self._max_depth,
-                num_leaves=self._num_leaves)
+                num_leaves=self._num_leaves,
+                min_child_samples=self._min_child_samples)
             return {
                 WidgetRequestResponseConstants.DATA: tree
             }
@@ -433,12 +461,14 @@ class ErrorAnalysisDashboardInput:
                 WidgetRequestResponseConstants.DATA: []
             }
 
-    def matrix(self, features, filters, composite_filters):
+    def matrix(self, features, filters, composite_filters,
+               quantile_binning, num_bins):
         try:
             if features[0] is None and features[1] is None:
                 return {WidgetRequestResponseConstants.DATA: []}
             matrix = self._error_analyzer.compute_matrix(
-                features, filters, composite_filters)
+                features, filters, composite_filters,
+                quantile_binning, num_bins)
             return {
                 WidgetRequestResponseConstants.DATA: matrix
             }
@@ -473,10 +503,10 @@ class ErrorAnalysisDashboardInput:
                 data = data.astype(dict(self._dfdtypes))
             if (self._is_classifier):
                 model_pred_proba = self._model.predict_proba(data)
-                prediction = self._convert_to_list(model_pred_proba)
+                prediction = _convert_to_list(model_pred_proba)
             else:
                 model_predict = self._model.predict(data)
-                prediction = self._convert_to_list(model_predict)
+                prediction = _convert_to_list(model_predict)
             return {
                 WidgetRequestResponseConstants.DATA: prediction
             }
@@ -487,18 +517,6 @@ class ErrorAnalysisDashboardInput:
                 WidgetRequestResponseConstants.DATA: []
             }
 
-    def _convert_to_list(self, array):
-        if issparse(array):
-            if array.shape[1] > 1000:
-                raise ValueError("Exceeds maximum number of "
-                                 "features for visualization (1000)")
-            return array.toarray().tolist()
-        if (isinstance(array, pd.DataFrame)):
-            return array.values.tolist()
-        if (isinstance(array, np.ndarray)):
-            return array.tolist()
-        return array
-
     def _find_first_explanation(self, key):
         interface = ExplanationDashboardInterface
         explanation_type_key = interface.MLI_EXPLANATION_TYPE_KEY
